/*
  WiFi RC Receiver
  2016-09-22 T. Nakagawa
*/

#include <ESP8266WiFi.h>
#include <Servo.h>
#include <Ticker.h>
#include <WiFiClient.h>
#include <WiFiUDP.h>

#define LED_PIN 4  // Active low
#define PWM_PINS {14, 12, 13, 5}
#define AP_SSID "AP0010"
#define AP_PASS "drondron"
#define PORT 12345
#define TIMEOUT 200  // (ms)

Ticker Flasher;
Ticker Watchdog;
Servo pwm[4];
WiFiUDP Udp;
const int Pwm_pin[] = PWM_PINS;
const int Channels = sizeof(Pwm_pin) / sizeof(*Pwm_pin);

void FlushLed() {
  static boolean flag = false;
  digitalWrite(LED_PIN, flag ? LOW : HIGH);
  flag = !flag;
}

void ResetPwm() {
  for (int i = 0; i < Channels; i++) {
    if (pwm[i].attached()) {
      pwm[i].detach();
      digitalWrite(Pwm_pin[i], LOW);
    }
  }
  Flasher.attach_ms(100, FlushLed);
}

void setup() {
  pinMode(LED_PIN, OUTPUT);
  digitalWrite(LED_PIN, HIGH);
  for (int i = 0; i < Channels; i++) {
    pinMode(Pwm_pin[i], OUTPUT);
    digitalWrite(Pwm_pin[i], LOW);
  }

  WiFi.mode(WIFI_AP);
  WiFi.softAP(AP_SSID, AP_PASS);
  Udp.begin(PORT);

  ResetPwm();
}

void loop() {
  if (Udp.parsePacket() > 0) {
    unsigned char buf[256];
    const int len = Udp.read(buf, 256);
    if (len < 2 + 2 * Channels || buf[0] < Channels) {
      return;
    }
    unsigned char chksum = 0;
    for (int i = 0; i < len; i++) {
      chksum += buf[i];
    }
    if (chksum != 0) {
      return;
    }

    for (int i = 0; i < Channels; i++) {
      const unsigned short val = (((unsigned short)buf[2 + 2 * i + 0] << 8) | buf[2 + 2 * i + 1]);
      const int width = (val < 32768) ? map(val, 0, 32768, 1000, 1500) : map(val, 32768, 65535, 1500, 2000);
      if (!pwm[i].attached()) {
        pwm[i].attach(Pwm_pin[i], 0, 2000);
      }
      pwm[i].writeMicroseconds(width);
    }
    Flasher.detach();
    digitalWrite(LED_PIN, LOW);
    Watchdog.once_ms(TIMEOUT, ResetPwm);
  }
}

